import {HttpErrorResponse} from "@angular/common/http";
import {
  ErrorHandler,
  Injectable,
  Injector,
} from "@angular/core";
import {ErrorHelper} from "../helpers/error.helper";
import {ENVIRONMENT} from "../injectors/environment.injector";
import {ErrorService} from "./error.service";
import {LoggingService} from "./logging.service";
import {NotificationService} from "./notification.service";

@Injectable({
  providedIn: "root",
})
export class GlobalErrorsHandlerService implements ErrorHandler {
  // Error handling is important and needs to be loaded first.
  // Because of this we should manually inject the services with Injector.
  constructor(protected injector: Injector) {
  }

  handleError(error: Error | HttpErrorResponse): void {
    const environment = this.injector.get(ENVIRONMENT);
    if (error === environment.errorToSkipInErrorHandler) {
      return;
    }
    const errorService = this.injector.get(ErrorService);
    const logger = this.injector.get(LoggingService);
    const notifier = this.injector.get(NotificationService);
    const httpErrorKeyToSkipInErrorHandler = environment.httpErrorKeyToSkipInErrorHandler;

    let message;
    let stackTrace;

    if (error instanceof HttpErrorResponse) {
      if (ErrorHelper.isErrorToTreat(error, httpErrorKeyToSkipInErrorHandler)) {
        // Server Error
        message = errorService.getServerMessage(error);
        stackTrace = errorService.getServerStack(error);
        notifier.showError(message, false);
      }
    } else {
      // Client Error
      message = errorService.getClientMessage(error);
      stackTrace = errorService.getClientStack(error);
      notifier.showError(message, false);
    }

    // Always log errors
    logger.logError(message, stackTrace, error);
  }
}
