import {
  Actions,
  Store,
} from "@ngxs/store";
import {ApiService} from "../../http/api.service";
import {
  BaseStateModel,
  BaseStoreNameSpace,
} from "../../models";
import {NotificationService} from "../../services";
import {isNullOrUndefined} from "../../tools";
import {StoreUtil} from "../../utils/stores/store.util";
import {BaseOptions} from "./base-options.model";

export const defaultBaseStateInitValue: () => BaseStateModel = () =>
  ({
    isLoadingCounter: 0,
  });

// @dynamic
export abstract class BaseState {
  protected abstract get _urlResource(): string;

  protected readonly _stateName: string;
  protected readonly _nameSpace: BaseStoreNameSpace;

  protected constructor(protected apiService: ApiService,
                        protected store: Store,
                        protected notificationService: NotificationService,
                        protected actions$: Actions,
                        protected options: BaseOptions,
                        protected baseStateConstructor: typeof BaseState | any) {
    this.options = Object.assign(baseStateConstructor.getDefaultOptions(), options);
    baseStateConstructor.checkOptions(this._stateName, this.options);
    this._stateName = StoreUtil.getState(this);
    this._nameSpace = options.nameSpace;
    StoreUtil.initState(baseStateConstructor, this.constructor, this._nameSpace);
  }

  // @InheritedSelector()
  // static isLoading(state: BaseStateModel): boolean {
  //   return StoreUtil.isLoadingState(state);
  // }

  protected static getDefaultOptions(): BaseOptions | any {
    return {};
  }

  protected static checkOptions(stateName: string, options: BaseOptions): void {
    const errorMessage = `Invalid options for resource state '${stateName}':`;
    if (isNullOrUndefined(options.nameSpace)) {
      throw new Error(`${errorMessage} resourceNameSpace should be defined`);
    }
  }
}
