import {
  isNullOrUndefined,
  isWhiteString,
} from "../tools";

export class ObjectUtil {
  static clone<T>(object: T): T {
    // Warning, it's not a deep copy !
    return Object.assign({}, object);
  }

  static getValue(object: any, key: string, defaultValue: any = undefined): any {
    if (isNullOrUndefined(object)) {
      return defaultValue;
    }
    return object[key];
  }

  static getNestedValue(object: any, nestedKey: string, defaultValue: any = undefined): any {
    if (isNullOrUndefined(nestedKey) || isWhiteString(nestedKey) || isNullOrUndefined(object)) {
      return defaultValue;
    }
    nestedKey = nestedKey.replace(/\[(\w+)\]/g, ".$1"); // convert indexes to properties
    nestedKey = nestedKey.replace(/^\./, "");           // strip a leading dot
    const listKey = nestedKey.split(".");
    for (let i = 0, n = listKey.length; i < n; ++i) {
      const key = listKey[i];
      if (key in object) {
        object = object[key];
      } else {
        return defaultValue;
      }
    }
    return object;
  }
}
