import {OrderApiEnum} from "../enums/order-api.enum";
import {OrderEnum} from "../enums/order.enum";
import {QueryParameters} from "../models/query-parameters/query-parameters.model";
import {isNullOrUndefined} from "../tools/is/is.tool";
import {ObjectUtil} from "./object.util";

export class QueryParametersUtil {
  public static getSortString(queryParameters: QueryParameters): string | null {
    if (isNullOrUndefined(queryParameters) || isNullOrUndefined(queryParameters.sort)) {
      return null;
    }
    return queryParameters.sort.field + "," + (queryParameters.sort.order === OrderEnum.descending ? OrderApiEnum.descending : OrderApiEnum.ascending);
  }

  public static getPageIndex(queryParameters: QueryParameters): number | null {
    if (isNullOrUndefined(queryParameters) || isNullOrUndefined(queryParameters.paging)) {
      return null;
    }
    return queryParameters.paging.pageIndex;
  }

  public static getPageSize(queryParameters: QueryParameters): number | null {
    if (isNullOrUndefined(queryParameters) || isNullOrUndefined(queryParameters.paging)) {
      return null;
    }
    return queryParameters.paging.pageSize;
  }

  public static resetToFirstPage(queryParameters: QueryParameters): QueryParameters {
    queryParameters = ObjectUtil.clone(queryParameters);
    if (queryParameters.paging.pageIndex) {
      const paging = ObjectUtil.clone(queryParameters.paging);
      paging.pageIndex = 0;
      queryParameters.paging = paging;
    }
    return queryParameters;
  }
}
